import React, { useState, useEffect } from 'react';
import { FaPlus, FaTrash, FaCalendarAlt, FaTimes, FaSave } from 'react-icons/fa';
import calendarService from '../services/CalendarService';
import './Calendar.sass';

const Calendar = () => {
  const [appointments, setAppointments] = useState({});
  const [users, setUsers] = useState([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState(null);
  const [currentDate, setCurrentDate] = useState(new Date());
  const [showModal, setShowModal] = useState(false);
  const [selectedDay, setSelectedDay] = useState(null);
  const [newAppointment, setNewAppointment] = useState({
    title: '',
    user: ''
  });

  useEffect(() => {
    loadUsers();
  }, []);

  useEffect(() => {
    loadAppointments();
  }, [currentDate]);

  const loadUsers = async () => {
    try {
      const userData = await calendarService.getUsers();
      setUsers(userData);
      if (userData.length > 0 && !newAppointment.user) {
        setNewAppointment(prev => ({ ...prev, user: userData[0].name }));
      }
    } catch (err) {
      setError('Failed to load users');
      console.error('Error loading users:', err);
    }
  };

  const loadAppointments = async () => {
    try {
      setLoading(true);
      const year = currentDate.getFullYear();
      const month = String(currentDate.getMonth() + 1).padStart(2, '0');
      
      const eventData = await calendarService.getEventsForMonth(year, month, null, 'family');
      
      // Group events by day
      const appointmentsByDay = {};
      eventData.forEach(event => {
        const day = parseInt(event.date.split('-')[2]);
        if (!appointmentsByDay[day]) {
          appointmentsByDay[day] = [];
        }
        appointmentsByDay[day].push({
          id: event.id,
          title: event.text,
          user: event.user,
          type: event.type
        });
      });
      
      setAppointments(appointmentsByDay);
      setError(null);
    } catch (err) {
      setError('Failed to load appointments');
      console.error('Error loading appointments:', err);
    } finally {
      setLoading(false);
    }
  };

  const handleAddAppointment = async (e) => {
    e.preventDefault();
    if (!newAppointment.title.trim() || !newAppointment.user) return;

    try {
      const year = currentDate.getFullYear();
      const month = String(currentDate.getMonth() + 1).padStart(2, '0');
      const day = String(selectedDay).padStart(2, '0');
      const date = `${year}-${month}-${day}`;

      const eventData = {
        user: newAppointment.user,
        date: date,
        text: newAppointment.title
      };

      const createdEvent = await calendarService.createEvent(eventData);
      
      // Update local state
      const newAppt = {
        id: createdEvent.id,
        title: newAppointment.title,
        user: newAppointment.user
      };

      setAppointments(prev => ({
        ...prev,
        [selectedDay]: [...(prev[selectedDay] || []), newAppt]
      }));

      // Reset form
      setNewAppointment({ title: '', user: users[0]?.name || '' });
      setShowModal(false);
      setSelectedDay(null);
    } catch (err) {
      setError('Failed to create appointment');
      console.error('Error creating appointment:', err);
    }
  };

  const handleDeleteAppointment = async (day, appointmentId, user) => {
    try {
      if (appointmentId.includes('birthday-')) {
        setError('Birthday events cannot be deleted');
        return;
      }

      await calendarService.deleteEvent(appointmentId, user);
      
      setAppointments(prev => ({
        ...prev,
        [day]: prev[day].filter(appt => appt.id !== appointmentId)
      }));
    } catch (err) {
      setError('Failed to delete appointment');
      console.error('Error deleting appointment:', err);
    }
  };

  const openAddModal = (day) => {
    setSelectedDay(day);
    setShowModal(true);
  };

  const closeModal = () => {
    setShowModal(false);
    setSelectedDay(null);
    setNewAppointment({ title: '', user: users[0]?.name || '' });
  };

  const goToToday = () => {
    setCurrentDate(new Date());
  };

  const navigateMonth = (direction) => {
    const newDate = new Date(currentDate);
    newDate.setMonth(newDate.getMonth() + direction);
    setCurrentDate(newDate);
  };

  const getDayName = (day) => {
    const date = new Date(currentDate.getFullYear(), currentDate.getMonth(), day);
    return date.toLocaleDateString('de-DE', { weekday: 'short' }).toUpperCase();
  };

  const isWeekend = (day) => {
    const date = new Date(currentDate.getFullYear(), currentDate.getMonth(), day);
    const dayOfWeek = date.getDay();
    return dayOfWeek === 0 || dayOfWeek === 6; // Sunday or Saturday
  };

  const isToday = (day) => {
    const today = new Date();
    return (
      day === today.getDate() &&
      currentDate.getMonth() === today.getMonth() &&
      currentDate.getFullYear() === today.getFullYear()
    );
  };

  const getCurrentMonthYear = () => {
    return currentDate.toLocaleDateString('en-US', {
      month: 'long',
      year: 'numeric'
    });
  };

  const getDaysInMonth = () => {
    return new Date(currentDate.getFullYear(), currentDate.getMonth() + 1, 0).getDate();
  };

  const getUserColor = (userName) => {
    const colors = [
      '#3b82f6', // blue
      '#10b981', // green
      '#f59e0b', // amber
      '#ef4444', // red
      '#8b5cf6', // purple
      '#06b6d4', // cyan
      '#ec4899', // pink
      '#84cc16'  // lime
    ];
    
    const userIndex = users.findIndex(user => user.name === userName);
    return colors[userIndex % colors.length];
  };

  if (loading) {
    return (
      <div className="calendar-container loading">
        <div className="loading-spinner">
          <FaCalendarAlt />
          <p>Loading calendar...</p>
        </div>
      </div>
    );
  }

  return (
    <div className="calendar-container">
      {/* Header */}
      <header className="calendar-header">
        <div className="header-content">
          <div className="month-navigation">
            <button 
              className="nav-btn prev"
              onClick={() => navigateMonth(-1)}
            >
              ‹
            </button>
            <h1 className="month-title">{getCurrentMonthYear()}</h1>
            <button 
              className="nav-btn next"
              onClick={() => navigateMonth(1)}
            >
              ›
            </button>
          </div>
          <button className="today-btn" onClick={goToToday}>
            Today
          </button>
        </div>
      </header>

      {/* Error Message */}
      {error && (
        <div className="error-message">
          {error}
          <button onClick={() => setError(null)}>×</button>
        </div>
      )}

      {/* Calendar Grid */}
      <main className="calendar-grid">
        {Array.from({ length: getDaysInMonth() }, (_, i) => {
          const day = i + 1;
          const dayAppointments = appointments[day] || [];
          
          return (
            <div 
              key={day} 
              className={`day-row ${isWeekend(day) ? 'weekend' : ''} ${isToday(day) ? 'today' : ''}`}
            >
              {/* Day Info Box */}
              <div className="day-info">
                <span className="day-number">{day}</span>
                <span className="day-name">{getDayName(day)}</span>
              </div>

              {/* Add Button */}
              <button 
                className="add-button"
                onClick={() => openAddModal(day)}
                title={`Add appointment for ${day}`}
              >
                <FaPlus />
              </button>

              {/* Appointments Container */}
              <div className="appointments-container">
                {dayAppointments.map((appointment, index) => (
                  <div 
                    key={appointment.id} 
                    className={`appointment-card ${appointment.type === 'birthday' ? 'birthday' : ''}`}
                    style={{ borderLeftColor: getUserColor(appointment.user) }}
                  >
                    <div className="appointment-content">
                      <div className="appointment-title">{appointment.title}</div>
                      <div className="appointment-user" style={{ color: getUserColor(appointment.user) }}>
                        {appointment.user}
                      </div>
                    </div>
                    {appointment.type !== 'birthday' && (
                      <button 
                        className="delete-btn"
                        onClick={() => handleDeleteAppointment(day, appointment.id, appointment.user)}
                        title="Delete appointment"
                      >
                        <FaTrash />
                      </button>
                    )}
                  </div>
                ))}
              </div>
            </div>
          );
        })}
      </main>

      {/* Add Appointment Modal */}
      {showModal && (
        <div className="modal-overlay" onClick={closeModal}>
          <div className="modal-content" onClick={(e) => e.stopPropagation()}>
            <div className="modal-header">
              <h3>Add Appointment - Day {selectedDay}</h3>
              <button className="close-btn" onClick={closeModal}>
                <FaTimes />
              </button>
            </div>
            
            <form onSubmit={handleAddAppointment} className="appointment-form">
              <div className="form-group">
                <label htmlFor="user">User</label>
                <select
                  id="user"
                  value={newAppointment.user}
                  onChange={(e) => setNewAppointment({ ...newAppointment, user: e.target.value })}
                  required
                >
                  {users.map(user => (
                    <option key={user.id} value={user.name}>
                      {user.name}
                    </option>
                  ))}
                </select>
              </div>

              <div className="form-group">
                <label htmlFor="title">Title</label>
                <input
                  type="text"
                  id="title"
                  placeholder="Enter appointment title..."
                  value={newAppointment.title}
                  onChange={(e) => setNewAppointment({ ...newAppointment, title: e.target.value })}
                  autoFocus
                  required
                />
              </div>

              <div className="form-actions">
                <button type="button" className="btn btn-secondary" onClick={closeModal}>
                  <FaTimes />
                  Cancel
                </button>
                <button type="submit" className="btn btn-primary">
                  <FaSave />
                  Save
                </button>
              </div>
            </form>
          </div>
        </div>
      )}
    </div>
  );
};

export default Calendar;
